@@ -1,5 +1,5 @@
 /*
- * fstype.c, by rmk
+ * by rmk
  *
  * Detect filesystem type (on stdin) and output strings for two
  * environment variables:
@@ -16,7 +16,6 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <string.h>
-#include <errno.h>
 #include <stdlib.h>
 #include "fstype.h"
 
@@ -37,7 +36,7 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
-	if (argc > 1 && argv[1][0] == '-' && argv[1][1] == '\0') {
+	if (argc > 1 && !(argv[1][0] == '-' && argv[1][1] == '\0')) {
 		fd = open(file = argv[1], O_RDONLY);
 		if (fd < 0) {
 			perror(argv[1]);
