@@ -103,8 +103,8 @@ static int run_linuxrc(int argc, char *argv[], dev_t root_dev)
 	fclose(fp);
 
 	mkdir("/old", 0700);
-	root_fd = open_cloexec("/", O_RDONLY | O_DIRECTORY, 0);
-	old_fd = open_cloexec("/old", O_RDONLY | O_DIRECTORY, 0);
+	root_fd = open("/", O_RDONLY|O_DIRECTORY|O_CLOEXEC, 0);
+	old_fd = open("/old", O_RDONLY|O_DIRECTORY|O_CLOEXEC, 0);
 
 	if (root_fd < 0 || old_fd < 0)
 		return -errno;
@@ -122,7 +122,8 @@ static int run_linuxrc(int argc, char *argv[], dev_t root_dev)
 		_exit(255);
 	} else if (pid > 0) {
 		dprintf("kinit: Waiting for linuxrc to complete...\n");
-		while (waitpid(pid, NULL, 0) != pid) ;
+		while (waitpid(pid, NULL, 0) != pid)
+			;
 		dprintf("kinit: linuxrc done\n");
 	} else {
 		return -errno;
