@@ -2,6 +2,6 @@
 #define RESUME_H
 
 int do_resume(int argc, char *argv[]);
-int resume(const char *resume_file);
+int resume(const char *resume_file, unsigned long long resume_offset);
 
 #endif /* RESUME_H */
