@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *
- *   Copyright 2004 H. Peter Anvin - All Rights Reserved
+ *   Copyright 2004-2006 H. Peter Anvin - All Rights Reserved
  *
  *   Permission is hereby granted, free of charge, to any person
  *   obtaining a copy of this software and associated documentation
@@ -32,7 +32,6 @@
  * do anything.
  */
 
-#include <alloca.h>
 #include <assert.h>
 #include <dirent.h>
 #include <errno.h>
@@ -65,7 +64,8 @@ static int nuke_dir(const char *what)
 	struct dirent *d;
 	int err = 0;
 
-	if (!(dir = opendir(what))) {
+	dir = opendir(what);
+	if (!dir) {
 		/* EACCES means we can't read it.  Might be empty and removable;
 		   if not, the rmdir() in nuke() will trigger an error. */
 		return (errno == EACCES) ? 0 : errno;
@@ -101,32 +101,21 @@ static int nuke(const char *what)
 			/* It's a directory. */
 			err = nuke_dir(what);
 			if (!err)
-				err = rmdir(what) ? errno : err;
-		} else {
-			err = errno;
+				rmdir(what);
 		}
 	}
 
-	if (err) {
-		fprintf(stderr, "%s: %s: %s\n", program, what, strerror(err));
-		return err;
-	} else {
-		return 0;
-	}
+	return 0;
 }
 
 int main(int argc, char *argv[])
 {
 	int i;
-	int err = 0;
 
 	program = argv[0];
 
-	for (i = 1; i < argc; i++) {
-		err = nuke(argv[i]);
-		if (err)
-			break;
-	}
+	for (i = 1; i < argc; i++)
+		nuke(argv[i]);
 
-	return !!err;
+	return 0;
 }
